
#include "TArtAnaLoopManager.hh"

#include <iostream>
#include <fstream>
#include <cstring>

#include "TThread.h"
#include "TDirectory.h"
#include "TList.h"
#include "TH1.h"
#include "TSystem.h"

#include "TArtAnaLoop.hh"
#include "TArtAnaFile.hh"

#include "TArtAnaClock.hh"

//_______________________________________________________________________________________


void book(TArtAnaLoop* analoop, const char* anafilename){TArtAnaLoopManager::Book(analoop, anafilename);}
void push(const char* filename, int eventnumber){TArtAnaLoopManager::PushRidf(filename, eventnumber);}
void pop(int i){TArtAnaLoopManager::PopRidf(i);}
void start(){TArtAnaLoopManager::Start();}
void stop(){TArtAnaLoopManager::Stop();}
void next(){TArtAnaLoopManager::Next();}
void end(){TArtAnaLoopManager::End();}
void clear(){TArtAnaLoopManager::ClearAll();}
void status(){TArtAnaLoopManager::Status();}

TCutG* hcut(int id, const char* filename, const char* cutname){return TArtAnaLoopManager::HCut(id, filename, cutname);}

//_______________________________________________________________________________________

TArtAnaLoopManager::EAnalysisState TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::INIT;
TThread* TArtAnaLoopManager::fThread = 0;
bool TArtAnaLoopManager::fLock = false;
bool TArtAnaLoopManager::fAnaFileLast = false;

std::deque<const char*> TArtAnaLoopManager::fNextRidfFileName;
std::deque<int> TArtAnaLoopManager::fNextRidfEventNumber;
std::deque<const char*> TArtAnaLoopManager::fPrevRidfFileName;

TThread* TArtAnaLoopManager::Book(TArtAnaLoop* analoop, const char* anafilename)
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(kAnalysisState == ERROR){
    std::cout << "\n recover from error\n" << std::endl;
    //    ClearAll();
    kAnalysisState = INIT;
  }

  if(fThread){
    if(kAnalysisState != INIT && kAnalysisState != END) End();
    delete fThread;
    fThread = 0;
  }

  if(kAnalysisState == END){
    kAnalysisState = INIT;
  }

  if(!gDirectory){
    std::cout << "\n gDirectory is not valid\n" << std::endl;
    return 0;
  }
  TArtAnaLoopStatic::SetDirectory(gDirectory);

  //  TArtAnaClock::Start();

  TArtAnaLoop* analoop_last = TArtAnaLoopStatic::GetAnaLoop();

  if(!analoop){
    if(analoop_last){
      if(anafilename && strcmp(anafilename, "")){      
	std::cout << "\n restart analoop by new anafile\n" << std::endl;
	if(!analoop_last->ChangeAnaFile(anafilename)) return 0;
      }else{
	/* nothing */;
      }
    }else{
      std::cout << "\n no analoop is set\n" << std::endl;
      return 0;
    }
    analoop = analoop_last;
  }else{
    if(analoop_last){
      if(fAnaFileLast) ClearHist(); // anafileを使わない場合はhistは自己管理
	
      if(anafilename && strcmp(anafilename, "")){      
	std::cout << "\n reset analoop and anafile\n" << std::endl;
	//ClearHist();
	delete analoop_last;
	analoop_last = 0;
	if(!analoop->OpenAnaFile(anafilename)) return 0;
      }else{
	std::cout << "\n reset analoop\n" << std::endl;
	//	ClearHist();
	delete analoop_last;
	analoop_last = 0;
      }
    }else{
      if(anafilename && strcmp(anafilename, "")){      
	if(!analoop->OpenAnaFile(anafilename)) return 0;
	std::cout << "\n set analoop and anafile\n" << std::endl;
      }else{
	/* nothing */
      }
    }
  }

  if(anafilename && strcmp(anafilename, "")){      
    fAnaFileLast = true;
  }else{
    fAnaFileLast = false;
  }

//  TArtAnaClock::Stop();
//  std::cout << "\n time:" << std::endl;
//  TArtAnaClock::Print();

  TArtAnaLoopStatic::SetAnaLoop(analoop);

  void *(*fpt)(void*) = &TArtAnaLoopStatic::AnaLoopStatic;

  return fThread = new TThread((void *(*)(void*))fpt, (void*)0);
}

void TArtAnaLoopManager::PushRidf(const char* filename, int eventnumber)
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  fNextRidfFileName.push_back(filename);
  fNextRidfEventNumber.push_back(eventnumber);

  std::cout << "\n Push Data File to: "
	    << fNextRidfFileName.size()
	    << std::endl;
  std::cout << std::endl;
}

void TArtAnaLoopManager::PopRidf(int i)
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(i>(int)fNextRidfFileName.size() || i<1){
    std::cout << "\n cannot pop " << i << "\n" << std::endl;
    return;
  }
    
  const char* filename = fNextRidfFileName[i-1];
  fNextRidfFileName.erase(fNextRidfFileName.begin()+i-1);
  fNextRidfEventNumber.erase(fNextRidfEventNumber.begin()+i-1);

  if(!filename || !strcmp(filename, "")) filename = "online";
  std::cout << "\n Pop Data File: "
	    << filename
	    << std::endl;
  std::cout << std::endl;
}

void TArtAnaLoopManager::Start()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n ...running\n" << std::endl;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n resume analoop\n" << std::endl;    
    kAnalysisState = RUNNING;
  }else if(kAnalysisState == NEXT){
    std::cout << "\n stack of data file is empty\n" << std::endl;    
  }else if(kAnalysisState == END){
    std::cout << "\n start analoop\n" << std::endl;
    kAnalysisState = INIT;
    fThread->Run();
  }else if(kAnalysisState == ERROR){
    std::cout << "\n Some error occur. Call \"book() or clear()\" to recover. \n" << std::endl;    
  }
}

void TArtAnaLoopManager::Stop()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n stop analoop\n" << std::endl;
    kAnalysisState = SUSPENDED;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n ...suspended\n" << std::endl;    
  }else if(kAnalysisState == NEXT){
    std::cout << "\n stop analoop\n" << std::endl;
    kAnalysisState = SUSPENDED;
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }else if(kAnalysisState == ERROR){
    std::cout << "\n Some error occur. Call \"book() or clear()\" to recover. \n" << std::endl;    
  }
}

void TArtAnaLoopManager::Next()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n analyze next data file\n" << std::endl;    
    kAnalysisState = NEXT;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n analyze next data file\n" << std::endl;    
    kAnalysisState = NEXT;
  }else if(kAnalysisState == NEXT){
    std::cout << "\n waiting for next data file\n" << std::endl;    
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }else if(kAnalysisState == ERROR){
    std::cout << "\n ...error\n" << std::endl;
  }
}

void TArtAnaLoopManager::End()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  if(!fThread){
    std::cout << "\n no anafile is set\n" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Join();
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Join();
  }else if(kAnalysisState == NEXT){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Join();
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
    fThread->Join();
  }else if(kAnalysisState == ERROR){
    std::cout << "\n Some error occur. Call \"book() or clear()\" to recover. \n" << std::endl;    
  }
}

void TArtAnaLoopManager::ClearAll()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  std::cout << "\n Clear All. \n" << std::endl;    

  if(fThread){
    if(kAnalysisState != END){
      kAnalysisState = END;
    }
    fThread->Join();
  }

  delete fThread;
  fThread = 0;

  fNextRidfFileName.clear();
  fPrevRidfFileName.clear();

  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  delete analoop;

  TArtAnaLoopStatic::Clear();

  ClearHist();

  fAnaFileLast = false;

  kAnalysisState = INIT;
}

void TArtAnaLoopManager::ClearHist()
{
  if(!gDirectory) return;
  TList* list = gDirectory->GetList();
  if(!list) return;
  TObjLink* objlink = list->FirstLink();	 
  TObject* obj;
  while(objlink){
    obj = objlink->GetObject();
    objlink = objlink->Next();
    if(obj && obj->InheritsFrom(TH1::Class())){
      delete obj;
    }
  }
}

void TArtAnaLoopManager::Status()
{
  if(fLock) if(gSystem) gSystem->Sleep(fSystemWait);

  std::cout << std::endl;

  TString state;
  switch (kAnalysisState){
  case INIT:
    state = "INIT"; break;
  case RUNNING:
    state = "RUNNING"; break;
  case SUSPENDED:
    state = "SUSPENDED"; break;
  case NEXT:
    state = "NEXT"; break;
  case END:
    state = "END"; break;
  case ERROR:
    state = "ERROR"; break;
  }

  const char* filename;

  std::cout << " Analysis State    : " << state << std::endl;
  if(kAnalysisState == INIT ||
     kAnalysisState == NEXT ||
     kAnalysisState == END){
    filename = "";
  }else{
    filename = TArtAnaLoopStatic::GetFileName();
  }
  std::cout << " File Name         : " << filename << std::endl;
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  TString classname("");
  int eventnumber(0);
  int totaleventnumber(0);
  if(analoop){
    classname = analoop->ClassName();
    eventnumber = analoop->GetEventNumber();
    totaleventnumber = analoop->GetTotalEventNumber();
  }
  std::cout << " Class Name        : " << classname << std::endl;
  std::cout << " Event Number      : " << eventnumber << std::endl;
  std::cout << " Total Event Number: " << totaleventnumber << std::endl;

  if(!fNextRidfFileName.empty()){
    std::cout << "\n Stack of Data File: " << std::endl;
    for(int i=0; i<(int)fNextRidfFileName.size(); ++i){
      filename = fNextRidfFileName[i];
      if(!filename || !strcmp(filename, "")) filename = "online";
      std::cout << "  " << i+1 << ": " << filename << std::endl;
    }
  }

  if(!fPrevRidfFileName.empty()){
    std::cout << "\n Analyzed Data File: " << std::endl;
    for(int i=0; i<(int)fPrevRidfFileName.size(); ++i){
      filename = fPrevRidfFileName[i];
      if(!filename || !strcmp(filename, "")) filename = "online";
      std::cout << "  " << i+1 << ": " << filename << std::endl;
    }
  }
  std::cout << std::endl;
}

TCutG* TArtAnaLoopManager::HCut(int id, const char* filename, const char* cutname)
{
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  if(!analoop) return 0;
  if(!analoop->fAnaFile) return 0;

  return analoop->fAnaFile->CreateCutG(id, filename, cutname);
}

TArtAnaLoopManager::~TArtAnaLoopManager()
{
  delete fThread;
  fThread = 0;
}

//_______________________________________________________________________________________

TArtAnaLoop* TArtAnaLoopStatic::fAnaLoop = 0;
TString TArtAnaLoopStatic::fFileName = "";
int TArtAnaLoopStatic::fEventNumber = -1;
TDirectory* TArtAnaLoopStatic::fDirectory = 0;

void* TArtAnaLoopStatic::AnaLoopStatic(void *)
{
  fDirectory->cd();
  if(fFileName.Length() == 0){
    fAnaLoop->AnaLoop();
  }else{
    fAnaLoop->AnaLoop();
  }    

  return 0;
}

void TArtAnaLoopStatic::SetAnaLoop(TArtAnaLoop* analoop)
{
  fAnaLoop = analoop;
}
void TArtAnaLoopStatic::SetFileName(const char* filename)
{
  if(!filename){
    fFileName = "";
  }else{
    fFileName = filename;
  }
}
void TArtAnaLoopStatic::SetEventNumber(int eventnumber){fEventNumber = eventnumber;}
void TArtAnaLoopStatic::SetDirectory(TDirectory* directory)
{
  if(!directory){
    fDirectory = gDirectory;
  }else{
    fDirectory = directory;
  }
}  
TArtAnaLoop* TArtAnaLoopStatic::GetAnaLoop(){return fAnaLoop;}
const char* TArtAnaLoopStatic::GetFileName(){return fFileName.Data();}
int TArtAnaLoopStatic::GetEventNumber(){return fEventNumber;}
TDirectory* TArtAnaLoopStatic::GetDirectory(){return fDirectory;}

void TArtAnaLoopStatic::Clear()
{
  fAnaLoop = 0;
  fFileName = "";
  fEventNumber = -1;
  fDirectory = 0;
}

TArtAnaLoopStatic::~TArtAnaLoopStatic()
{
  Clear();
}

