#ifndef TARTALIAS_HH
#define TARTALIAS_HH

#include "TObject.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

class TVirtualPad;
class TH1;
class TH1D;
class TH2;
class TH3;
class TProfile;

//Read all TH1 object in file(filename) to "current directory".
void fetch(char* filename);
//Write all TH1 object in current directory to file(filename)
void hstore(char* filename);
//Delete all TH1 object in current directory.
void hdel();
//delete hist by id
void hdel(int id);
//delete hist between idstart and idend
void hdel(int idstart, int idend);

//show list of hist in current directory.
void ls();

//move to nexd pad and draw hist from idstart to idend
void ht(int idstart, int idend);
//move to nexd pad and draw hist by id.
void ht(int id);
//move to nexd pad and draw current hist.
void htp();
//move to nexd pad and draw next hist.
void hn();
//move to nexd pad and draw previous hist.
void hb();

//draw hist from idstart to idend
void hht(int idstart, int idend);
//draw hist by id.
void hht(int id);
//draw current hist.
void hhtp();
//draw next hist.
void hhn();
//draw previous hist.
void hhb();

//set xaxis of gPad to logscale
void lgx();
//set yaxis of gPad to logscale
void lgy();
//set zaxis of gPad to logscale
void lgz();
//unset lgx
void lnx();
//unset lgy
void lny();
//unset lgz
void lnz();
//do lgx, lgy, and lgz
void lg();
//do lnx, lny, and lnz
void ln();

//change size of canvas
void size(UInt_t ww, UInt_t wh);
//cd to subpadnumber(0 is assigned to root canvas)
void cd(Int_t subpadnumber = 0);
//cd to next subpad
void cdn();
//cd to previous subpad
void cdb();
//divide canvas(not gPad).
void zone(Int_t nx = 1, Int_t ny = 1, Float_t xmargin = 0.01, Float_t ymargin = 0.01, Int_t color = 0);

//gui command
Double_t xval();
Double_t yval();
void xyval();
void zoom();
void unzoom();
void blow();//same as xyblow of anapaw
//status of hist
void hsta();
//status of selected reigon
void xsta();
//cut out selected region of TH2
void cpro();
//graphical fit
TFitResultPtr xfitp(int n = 1);
TFitResultPtr xfite();
TFitResultPtr xfitg();
TFitResultPtr xfitl();

//same as fitg()
void figa();
TFitResultPtr fit(const char* formula, Option_t* option = "", Option_t* goption = "", Double_t xmin = 0, Double_t xmax = 0);
TFitResultPtr fitp(int n = 1, Double_t xmin = 0, Double_t xmax = 0, Option_t* option = "", Option_t* goption = "");
TFitResultPtr fite(Double_t xmin = 0,  Double_t xmax = 0, Option_t* option = "", Option_t* goption = "");
TFitResultPtr fitg(Double_t xmin = 0,  Double_t xmax = 0, Option_t* option = "", Option_t* goption = "");
TFitResultPtr fitl(Double_t xmin = 0,  Double_t xmax = 0, Option_t* option = "", Option_t* goption = "");

//change bin
void chbin(Int_t ngroupx, Int_t ngroupy = 1, Int_t ngroupz = 1);

//profile
void pfx();
void pfy();
//projection
void prx();
void pry();
//slice
void slx(Int_t num);
void sly(Int_t num);
//ban
void bnx(double firsty, double lasty, Option_t* option = "");
void bny(double firstx, double lastx, Option_t* option = "");
//blow
void blowxyz(double firstx = 0, double lastx = -1,
	     double firsty = 0, double lasty = -1,
	     double firstz = 0, double lastz = -1);
void blowx(double firstx = 0, double lastx = -1);
void blowy(double firsty = 0, double lasty = -1);
void blowz(double firstz = 0, double lastz = -1);

//operations on histograms
void scale(Double_t c);
void add(Int_t id1, Int_t id2, Double_t c1=1, Double_t c2=1);
void multiply(Int_t id1, Int_t id2, Double_t c1=1, Double_t c2=1);
void divide(Int_t id1, Int_t id2, Double_t c1=1, Double_t c2=1);

//min max
void mami(double min, double max);

//print canvas
void print(char* filename = "", Option_t* option = "");

//___________________________________________________________________________
//not well developed functions

//read all object in file to current directory
void readobject(char* filename);
//write all object in current directory to root file
void writeobject(char* filename);
//write hist contents to text
void writehisttotext(char* filename);
//create hist from text. only text which created from writehist is supported.
void createhist(char* filename);
//create graph from text.
void creategraph(char* filename);
//return hist of id
TH1* gethist(int id);
//set default option of hist
void setoption(Option_t* option);

//like watch command
//void watch(void (*fpt)(), UInt_t milliSec, Int_t num);

// TH1* fillhist(const char* name, const char* title,
// 	      Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup);
// TH2* fillhist(const char* name, const char* title,
// 	      Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup,
// 	      Double_t valy, Int_t nbinsy, Double_t ylow, Double_t yup);
// TH3* fillhist(const char* name, const char* title,
// 	      Double_t valx, Int_t nbinsx, Double_t xlow, Double_t xup,
// 	      Double_t valy, Int_t nbinsy, Double_t ylow, Double_t yup,
// 	      Double_t valz, Int_t nbinsz, Double_t zlow, Double_t zup);
// TH1* fillhist(const char* name, const char* title, Int_t size,
// 	      Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup);
// TH2* fillhist(const char* name, const char* title, Int_t size,
// 	      Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup,
// 	      Double_t* valy, Int_t nbinsy, Double_t ylow, Double_t yup);
// TH3* fillhist(const char* name, const char* title, Int_t size,
// 	      Double_t* valx, Int_t nbinsx, Double_t xlow, Double_t xup,
// 	      Double_t* valy, Int_t nbinsy, Double_t ylow, Double_t yup,
// 	      Double_t* valz, Int_t nbinsz, Double_t zlow, Double_t zup);


#endif
